@@ -245,11 +245,11 @@ class SYNOP(pymetdecoder.Report):
                         elif i == 1: # Period and height of waves (instrumental)
                             if "wind_waves" not in data:
                                 data["wind_waves"] = []
-                            data["wind_waves"].append(obs.WindWaves().decode(next_group, instrumental=True))
+                            data["wind_waves"].append(obs.WindWaves().decode(next_group, instrumental=True, waves=data["wind_waves"]))
                         elif i == 2: # Period and height of wind waves
                             if "wind_waves" not in data:
                                 data["wind_waves"] = []
-                            data["wind_waves"].append(obs.WindWaves().decode(next_group, instrumental=False))
+                            data["wind_waves"].append(obs.WindWaves().decode(next_group, instrumental=False, waves=data["wind_waves"]))
                         elif i == 3: # Swell wave directions
                             sw_dirs = next_group
                         elif i == 4 or i == 5:
@@ -263,7 +263,27 @@ class SYNOP(pymetdecoder.Report):
                         elif i == 7: # Accurate wave height
                             if "wind_waves" not in data:
                                 data["wind_waves"] = []
-                            data["wind_waves"].append(obs.WindWaves().decode(next_group, instrumental=True))
+
+                            # First, find the existing instrumental wave measurement (from group 1)
+                            # We need this, otherwise this group is superfluous
+                            instrumental = None
+                            for w in data["wind_waves"]:
+                                if w["instrumental"]:
+                                    instrumental = w
+                                    break
+                            if instrumental is None:
+                                logging.warning("1pphh group required if 70hhh group is specified")
+                                continue
+
+                            # Next, check the inaccurate (group 1) height is similar to the accurate
+                            # measurement in this group. If not, warn
+                            this_wave = obs.WindWaves().decode(next_group, instrumental=False, waves=data["wind_waves"])
+                            if not (instrumental["height"]["value"] - 0.5 <= this_wave["height"]["value"] <= instrumental["height"]["value"] + 0.5):
+                                logging.warning("Differing heights for wind wave between group 1 and group 7")
+
+                            # Update the instrumental wave height with the accurate version
+                            instrumental["height"] = this_wave["height"]
+                            instrumental["accurate"] = True
                         elif i == 8:
                             data["wet_bulb_temperature"] = obs.WetBulbTemperature().decode(next_group)
                         next_group = next(groups)
@@ -280,11 +300,12 @@ class SYNOP(pymetdecoder.Report):
             group_9 = []
             group_5 = None
             group_6 = 0
+            msg_5   = []
             if next_group == "333":
                 next_group = next(groups)
                 last_header = None
                 while True:
-                    if re.match("^(555)$", next_group):
+                    if re.match("^(444)$", next_group) or re.match("^(555)$", next_group):
                         break
                     try:
                         header = int(next_group[0])
@@ -296,40 +317,9 @@ class SYNOP(pymetdecoder.Report):
                         break
 
                     if (0 <= header <= 6) and group_5 is not None:
-                        # Determine if we're expecting group 3 precipitation
-                        if next_group.startswith("6") and data["precipitation_indicator"]["in_group_3"]:
-                            data["precipitation_s3"] = obs.Precipitation().decode(next_group, tenths=False)
-                            # group_5 = None
-                            group_6 += 1
-
-                            # print(data["precipitation_indicator"]["in_group_3"])
-
-                        if group_5[2] == "3":
-                            radiation_time = { "value": 1, "unit": "h" }
-                            radiation_unit = "kJ/m2"
-                        elif group_5[1] == "5":
-                            radiation_time = { "value": 24, "unit": "h" }
-                            radiation_unit = "J/cm2"
-                        radiation = obs.Radiation().decode(next_group[1:5],
-                            unit = radiation_unit,
-                            time_before = radiation_time
-                        )
-                        matches = re.match("55[45]0([78])", group_5)
-                        if matches:
-                            if matches.group(1) == "7":
-                                radiation_type = "net_short_wave"
-                            else:
-                                radiation_type = "direct_solar"
-                        else:
-                            radiation_type = RADIATION_TYPES[header]
-                        group_5 = None
-                        if "radiation" not in data:
-                            data["radiation"] = {}
-                        if radiation_type not in data["radiation"]:
-                            data["radiation"][radiation_type] = []
-                        data["radiation"][radiation_type].append(radiation)
-                        # logging.warning("parse group 5: {} (last: {})".format(next_group, group_5))
-                        # group_5 = None
+                        # Append sunshine and radiation groups to a separate list
+                        # to deal with later
+                        msg_5.append(next_group)
                     else:
                         if header == 0:
                             if data["region"] is None:
@@ -366,17 +356,18 @@ class SYNOP(pymetdecoder.Report):
                                     data["temperature_change"] = obs.TemperatureChange().decode(next_group[2:5])
                                 elif j[1] == "5": # 55xxx
                                     if j[2] in ["0", "1", "2", "3"]: # 55[0123]xx
-                                        data["sunshine"] = obs.Sunshine().decode(next_group)
                                         group_5 = next_group
                                     elif j[2] in ["4", "5"]: # 55[45]xx
                                         if next_group[3:5] not in ["07", "08"]:
                                             raise pymetdecoder.InvalidCode(next_group, "5jjjj")
                                         group_5 = next_group
                                     elif j[2] == "/": # 55/xx
-                                        data["sunshine"] = obs.Sunshine().decode(next_group)
+                                        # This is fine, but we don't want to raise an error
+                                        pass
                                     else:
                                         raise pymetdecoder.InvalidCode(next_group, "section 3 group 5")
                                     group_5 = next_group
+                                    msg_5.append(group_5)
                                 elif j[1] in ["6"]: # 56xxx
                                     data["cloud_drift_direction"] = obs.CloudDriftDirection().decode(next_group)
                                 elif j[1] in ["7"]: # 57xxx
@@ -420,9 +411,9 @@ class SYNOP(pymetdecoder.Report):
 
                 # Remove unneeded radiation groups. This is not an elegant solution,
                 # but it works
-                if group_6 == 1:
-                    if "precipitation_s3" in data and "radiation" in data:
-                        del(data["radiation"])
+                # if group_6 == 1:
+                #     if "precipitation_s3" in data and "radiation" in data:
+                #         del(data["radiation"])
 
             # Parse group 9 before moving on
             if len(group_9) > 0:
@@ -431,13 +422,13 @@ class SYNOP(pymetdecoder.Report):
 
             ### SECTION 4 ###
             if next_group == "444":
-                data["section4"] = []
+                data["cloud_base_below_station"] = []
                 next_group = next(groups)
-                last_header = None
+                # last_header = None
                 while True:
                     if re.match("^(555)$", next_group):
                         break
-                    data["section4"].append(next_group)
+                    data["cloud_base_below_station"].append(obs.CloudBaseBelowStationLevel().decode(next_group))
                     next_group = next(groups)
             else:
                 if next_group != "555":
@@ -462,6 +453,47 @@ class SYNOP(pymetdecoder.Report):
                     data["sea_land_ice"] = obs.SeaLandIce().decode(ice_groups)
                 if len(group_9) > 0:
                     data = self._parse_group_9(data, group_9, def_time_before)
+
+                # Interpret radiation, sunshine and precipitation from section 3
+                for m in msg_5:
+                    if m.startswith("55"):
+                        g5 = m
+                        if "sunshine" not in data:
+                            data["sunshine"] = []
+                        data["sunshine"].append(obs.Sunshine().decode(m))
+                    else:
+                        if g5[2] == "3":
+                            radiation_time = { "value": 1, "unit": "h" }
+                            radiation_unit = "kJ/m2"
+                        elif g5[1] == "5":
+                            radiation_time = { "value": 24, "unit": "h" }
+                            radiation_unit = "J/cm2"
+                        radiation = obs.Radiation().decode(m[1:5],
+                            unit = radiation_unit,
+                            time_before = radiation_time
+                        )
+                        matches = re.match("55[45]0([78])", g5)
+                        if matches:
+                            if matches.group(1) == "7":
+                                radiation_type = "net_short_wave"
+                            else:
+                                radiation_type = "direct_solar"
+                        else:
+                            radiation_type = RADIATION_TYPES[int(m[0])]
+                        if "radiation" not in data:
+                            data["radiation"] = {}
+                        if radiation_type not in data["radiation"]:
+                            data["radiation"][radiation_type] = []
+                        data["radiation"][radiation_type].append(radiation)
+                if len(msg_5) > 0 and msg_5[-1].startswith("6") and data["precipitation_indicator"]["in_group_3"]:
+                    data["precipitation_s3"] = obs.Precipitation().decode(next_group, tenths=False)
+                    if "short_wave" in data["radiation"]:
+                        if len(data["radiation"]["short_wave"]) == 1:
+                            del(data["radiation"]["short_wave"])
+                        else:
+                            del(data["radiation"]["short_wave"][-1])
+                    # if len(data["radiation"]["short_wave"]):
+                    #     del(data["radiation"]["short_wave"][-1])
             except UnboundLocalError as e:
                 pass
             # return data
@@ -636,14 +668,15 @@ class SYNOP(pymetdecoder.Report):
         if "temperature_change" in data:
             s3_groups.append(obs.TemperatureChange().encode(data["temperature_change"], group="54"))
         if "sunshine" in data:
-            sunshine = obs.Sunshine().encode(data["sunshine"], group="55")
-            s3_groups.append(sunshine)
-            if "radiation" in data:
-                radiation_time = 1 if sunshine[3] == "3" else 24
-                for r, rad in data["radiation"].items():
-                    for x in rad:
-                        if "time_before_obs" in x and x["time_before_obs"]["value"] == radiation_time:
-                            s3_groups.append(obs.Radiation().encode(x, group=str(RADIATION_TYPES.index(r))))
+            for s in data["sunshine"]:
+                sunshine = obs.Sunshine().encode(s, group="55")
+                s3_groups.append(sunshine)
+                if "radiation" in data:
+                    radiation_time = 1 if sunshine[2] == "3" else 24
+                    for r, rad in data["radiation"].items():
+                        for x in rad:
+                            if "time_before_obs" in x and x["time_before_obs"]["value"] == radiation_time:
+                                s3_groups.append(obs.Radiation().encode(x, group=str(RADIATION_TYPES.index(r))))
         if "radiation" in data and "sunshine" not in data:
             for r, rad in data["radiation"].items():
                 for x in rad:
@@ -688,6 +721,8 @@ class SYNOP(pymetdecoder.Report):
                 s3_groups.append(obs.TimeBeforeObs().encode(data["weather_info"]["time_before_obs"], group="900"))
             if "variability" in data["weather_info"]:
                 s3_groups.append(obs.VariableLocationIntensity().encode(data["weather_info"]["variability"], group="900"))
+            if "time_of_ending" in data["weather_info"]:
+                s3_groups.append(obs.TimeOfEnding().encode(data["weather_info"]["time_of_ending"], group="901"))
             if "non_persistent" in data["weather_info"]:
                 s3_groups.append(obs.TimeBeforeObs().encode(data["weather_info"]["non_persistent"], group="905"))
         if "precipitation_begin" in data:
@@ -763,7 +798,8 @@ class SYNOP(pymetdecoder.Report):
         if "optical_phenomena" in data:
             s3_groups.append(obs.OpticalPhenomena().encode(data["optical_phenomena"], group="990"))
         if "mirage" in data:
-            s3_groups.append(obs.Mirage().encode(data["mirage"], group="991"))
+            for m in data["mirage"]:
+                s3_groups.append(obs.Mirage().encode(m, group="991"))
         if "st_elmos_fire" in data:
             s3_groups.append("99190")
         if "condensation_trails" in data:
@@ -785,6 +821,12 @@ class SYNOP(pymetdecoder.Report):
         if len(s3_groups) > 0:
             groups.extend(["333"] + s3_groups)
 
+        ### SECTION 4
+        if "cloud_base_below_station" in data:
+            groups.append("444")
+            for d in data["cloud_base_below_station"]:
+                groups.append(obs.CloudBaseBelowStationLevel().encode(d))
+
         ### SECTION 5
         if "section5" in data:
             groups.extend(["555"] + data["section5"])
@@ -1023,7 +1065,9 @@ class SYNOP(pymetdecoder.Report):
                     if g[3:5] == "90":
                         data["st_elmos_fire"] = True
                     else:
-                        data["mirage"] = obs.Mirage().decode(g)
+                        if "mirage" not in data:
+                            data["mirage"] = []
+                        data["mirage"].append(obs.Mirage().decode(g))
                 elif j[2] == "2":
                     data["condensation_trails"] = obs.CondensationTrails().decode(g)
                 elif j[2] == "3":
