@@ -121,6 +121,33 @@ class Callsign(Observation):
             raise InvalidCode(callsign, "callsign")
     def _encode(self, data):
         return str(data["value"]).upper()
+class CloudBaseBelowStationLevel(Observation):
+    """
+    Clouds with tops below station level (section 4)
+    """
+    _CODE_LEN = 5
+    class Altitude(Observation):
+        _CODE_LEN = 2
+        def _decode(self, HH):
+            return {
+                "value": int(HH) * 100,
+                "quantifier": "isGreaterOrEqual" if int(HH) == 99 else None,
+                "unit": "m"
+            }
+        def _encode(self, data):
+            value = int(data["value"] / 100)
+            if value > 99:
+                value = 99
+            return "{:02d}".format(value)
+    class Description(Observation):
+        _CODE_LEN = 1
+        _CODE_TABLE = ct.CodeTable0552
+    _COMPONENTS = [
+        ("cloud_cover", 0, 1, CloudCover),
+        ("genus", 1, 1, CloudGenus),
+        ("upper_surface_altitude", 2, 2, Altitude),
+        ("description", 4, 1, Description)
+    ]
 class CloudDriftDirection(Observation):
     """
     Direction of cloud drift
@@ -546,9 +573,9 @@ class ImportantWeather(Observation):
     def _decode(self, raw, **kwargs):
         use_4687 = kwargs.get("use_4687", False)
         if use_4687:
-            self._CODE_TABLE = ct.CodeTable4687
+            return ct.CodeTable4687().decode(raw, **kwargs)
         else:
-            return { "value": int(raw), "_table": "4677" }
+            return { "value": int(raw), "_table": "4677", "time_before_obs": kwargs.get("time_before") }
 class LocalPrecipitation(Observation):
     """
     Precipitation character and time of precipitation for Region I
@@ -617,7 +644,7 @@ class MountainCondition(Observation):
         _CODE_TABLE = ct.CodeTable2863
         _CODE_LEN = 1
     _COMPONENTS = [
-        ("conditions", 3, 1, Condition),
+        ("condition", 3, 1, Condition),
         ("evolution", 4, 1, Evolution)
     ]
 class ObservationTime(Observation):
@@ -1100,6 +1127,8 @@ class StationPosition(Observation):
         lat = raw[2:5]  # Latitude
         Q   = raw[6:7]  # Quadrant
         lon = raw[7:11] # Longitude
+        if Q not in ["1", "3", "5", "7"]:
+            raise InvalidCode(Q, "quadrant")
 
         # Check both values are numeric, otherwise we can't get the position
         try:
@@ -1186,14 +1215,14 @@ class StationPosition(Observation):
     class Latitude(Observation):
         def _decode(self, raw, **kwargs):
             quadrant = kwargs.get("quadrant")
-            return "{:.1f}".format(int(raw) / (-10.0 if quadrant in ["3", "5"] else 10.0))
+            return float("{:.1f}".format(int(raw) / (-10.0 if quadrant in ["3", "5"] else 10.0)))
         def _encode(self, data, **kwargs):
             quadrant = kwargs.get("quadrant")
             return int(float(data) * (-10.0 if quadrant in ["3", "5"] else 10.0))
     class Longitude(Observation):
         def _decode(self, raw, **kwargs):
             quadrant = kwargs.get("quadrant")
-            return "{:.1f}".format(int(raw) / (-10.0 if quadrant in ["5", "7"] else 10.0))
+            return float("{:.1f}".format(int(raw) / (-10.0 if quadrant in ["5", "7"] else 10.0)))
         def _encode(self, data, **kwargs):
             quadrant = kwargs.get("quadrant")
             return int(float(data) * (-10.0 if quadrant in ["5", "7"] else 10.0))
